function mus = MusFromMaterials(materials,energies,varargin)
%    function mus = MusFromMaterials(materials,energies)
%    returns the x-ray mass attenuation coefficients of materials,
%    which can be body materials such as 'adipose' tissue
%    or chemical formulas such as 'H2O'
%     inputs:
%         materials: a cell array of strings. Body materials implemented are:
%              'adipose','blood','bone_compact','bone_cortical', 'brain','lung',...
%              'muscle_skeletal','muscle_striated','skin','soft_tissue', 'water'
%              or a chemical formula such as 'H2O'. See help for xraymu.m 
%              for details on allowable formulas
%          energies: the x-ray energies in keV (kilo-electron-volts)
%     outputs:
%         mus: a (numel(energies) x length(materials) ) matrix of the mass attenuation coefficients
%     example:
%       materials = {'bone_compact','soft_tissue','I'};
%       mus = MusFromMaterials(materials,10:1:100);
%          % returns a 91,3 matrix of the mass attenuation coefficients
%    COPYRIGHT:  Robert E. Alvarez, 2014-2015. All rights reserved.

nreqargs = 2;
if(nargin<nreqargs), error('%d args required, only have %d',nreqargs,nargin);end

types = {'attenuation','absorption'};
mu_type = types{1}; % default attenuation: do this to be sure a valid type

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'mu_type'; mu_type = varargin{i+1};   validatestring(mu_type,types);  i=i+1;
    otherwise
        error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end
    % handle string or cell array of strings inputs
if iscell(materials)
    for k = 1:length(materials)
        assert(ischar(materials{k}));
    end
elseif ischar(materials)
    materials = {materials}; % make it into cell array
else
    error('materials has to be string or cell array of strings');
end
nmaterials = length(materials);
mus = zeros(numel(energies),nmaterials);
    
  % test for energies is 1d VECTOR
assert(isa(energies,'double') && isvector(energies));

bdat = BodyMaterialComposition;  % load the composition tables--local function see below
%  fields of bdat are: 'adipose','blood','bone_compact','bone_cortical', ...
% 'brain','lung','muscle_skeletal','muscle_striated','skin','soft_tissue' ...

for kmaterial = 1:nmaterials
    material = char(materials{kmaterial});
    if isfield(bdat,material)
        dz = bdat.(material); % bdat is struct from BodyMaterialCompositionFunc so this a ztable
        mus(:,kmaterial) = xraymu(dz(2:end,:),energies,'mu_type',mu_type)'; % the first row of dz has the density and is not used by xraymu
    else
        mus(:,kmaterial) = xraymu(material,energies,'mu_type',mu_type); % xraymu will throw error if material not OK chemical formula
    end
end


   % *************** LOCAL FUNCTIONS ***************
   
function dat = BodyMaterialComposition
%    function dat = BodyMaterialComposition
%    returns a struct with fields composition tables for some body materials
%    from http://physics.nist.gov/cgi-bin/Star/compos.pl?matno=123
%    inputs:
%      none
%    outputs:
%      dat: a struct with fields
%         adipose
%         blood
%         bone_compact  
%         bone_cortical
%         brain
%         lung
%         muscle_skeletal
%         muscle_striated
%         skin
%         soft_tissue
%         water
%    each entry is a 2 column matrix:
%         the first row has the density in the second column
%         the following rows have the atomic number in the first column
%         and the fraction by weight in the second column


d = [ ... % fat
-1	0.92   % -1=> density
1 	0.119477
6 	0.637240
7 	0.007970
8 	0.232333
11 	0.000500
12 	0.000020
15 	0.000160
16 	0.000730
17 	0.001190
19 	0.000320
20 	0.000020
26 	0.000020
30 	0.000020
];
dat.adipose = d;

d = [...
-1	1.06
1 	0.101866
6 	0.100020
7 	0.029640
8 	0.759414
11 	0.001850
12 	0.000040
14 	0.000030
15 	0.000350
16 	0.001850
17 	0.002780
19 	0.001630
20 	0.000060
26 	0.000460
30 	0.000010
];
dat.blood = d;

d = [ ...
-1	1.85
1 	0.063984
6 	0.278000
7 	0.027000
8 	0.410016
12 	0.002000
15 	0.070000
16 	0.002000
20 	0.147000
];
dat.bone_compact = d;

d = [ ...
-1	1.85
1 	0.047234
6 	0.144330
7 	0.041990
8 	0.446096
12 	0.002200
15 	0.104970
16 	0.003150
20 	0.209930
30 	0.000100
];
dat.bone_cortical = d;

d = [ ...
1 	0.110667
6 	0.125420
7 	0.013280
8 	0.737723
11 	0.001840
12 	0.000150
15 	0.003540
16 	0.001770
17 	0.002360
19 	0.003100
20 	0.000090
26 	0.000050
30 	0.000010
];
dat.brain = d;

d = [ ...
-1	1.05
1 	0.101278
6 	0.102310
7 	0.028650
8 	0.757072
11 	0.001840
12 	0.000730
15 	0.000800
16 	0.002250
17 	0.002660
19 	0.001940
20 	0.000090
26 	0.000370
30 	0.000010
];
dat.lung = d;

d = [ ...
-1	1.04
1 	0.100637
6 	0.107830
7 	0.027680
8 	0.754773
11 	0.000750
12 	0.000190
15 	0.001800
16 	0.002410
17 	0.000790
19 	0.003020
20 	0.000030
26 	0.000040
30 	0.000050
];
dat.muscle_skeletal = d;

d = [ ...
-1	1.04
1 	0.101997
6 	0.123000
7 	0.035000
8 	0.729003
11 	0.000800
12 	0.000200
15 	0.002000
16 	0.005000
19 	0.003000
];
dat.muscle_striated = d;


d = [ ...
-1	1.1
1 	0.100588
6 	0.228250
7 	0.046420
8 	0.619002
11 	0.000070
12 	0.000060
15 	0.000330
16 	0.001590
17 	0.002670
19 	0.000850
20 	0.000150
26 	0.000010
30 	0.000010
];
dat.skin = d;

d = [
-1	1.0
1 	0.104472
6 	0.232190
7 	0.024880
8 	0.630238
11 	0.001130
12 	0.000130
15 	0.001330
16 	0.001990
17 	0.001340
19 	0.001990
20 	0.000230
26 	0.000050
30 	0.000030
];
dat.soft_tissue = d;

d = [
-1	1.0
1 	0.1118944
8 	0.8881056
];
dat.water = d;

   